import zope.schema
from zope.interface import alsoProvides
from plone.app.batch.browser import fields
from plone.app.batch.vocabularies import TitledVocabulary

atfield_types = {'file': zope.schema.Bytes,
                 'datetime': zope.schema.Datetime,
                 'lines': fields.Lines,
                 'integer': zope.schema.Int,
                 'float': zope.schema.Float,
                 'fixedpoint': zope.schema.Decimal,
                 'reference': fields.Reference,
                 'computed': None,
                 'boolean': zope.schema.Bool,
                 'object': zope.schema.Object,
                 'image': zope.schema.Bytes,
                 'string': zope.schema.TextLine,
                 'text': zope.schema.Text}

def AT2Z3Field((atfield,objects)):
    """Returns a zope.schema field apropiated for the Archetype field given"""
    fieldType = atfield_types.get(atfield.type,None)
    if fieldType is None:
        raise TypeError('Could not adapt',(atfield,objects))
    name = atfield.getName()
    zfield = fieldType(title=unicode(name),
                       __name__=str(name),
                       required=atfield.required,
                       readonly='w' not in atfield.mode)
    for ob in objects:
        if ob.getField(name) is not None:
            vocab_items = atfield.Vocabulary(ob).items()
            if not vocab_items:
                continue
            vocab = TitledVocabulary.fromTitles(vocab_items)
            zfield.value_type = zope.schema.Choice(source=vocab)
            break

    return zfield
