from Acquisition import aq_inner

from z3c.form import form, field, button
from z3c.form.interfaces import INPUT_MODE, NOVALUE, IDataManager
from z3c.form.datamanager import DataManager
from plone.app.z3cform.layout import FormWrapper

from zope.app import zapi
from zope.component import adapts, getMultiAdapter
from zope.interface import alsoProvides, Interface

from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.Five.utilities import marker
from Products.statusmessages.interfaces import IStatusMessage

from plone.app.batch.browser.interfaces import IPloneBatchManagementContentTypes
from plone.app.batch.browser.widgets import ContentTypesMultiCheckBoxWidgetFactory
from plone.app.batch.interfaces import IBatchEditableContentTypes
from plone.app.batch.interfaces import IBatchEditableMarker
from plone.app.batch.browser.interfaces import IBatchContentTypesField
from plone.app.batch.browser import baseform
from plone.app.batch import BatchMessageFactory as _

class ManagePloneBatchBase(form.Form,baseform.BatchBaseForm):
    fields = field.Fields(IPloneBatchManagementContentTypes)
    fields['batch_editable_types'].widgetFactory[INPUT_MODE] = ContentTypesMultiCheckBoxWidgetFactory
    alsoProvides(fields['batch_editable_types'].field,
                     IBatchContentTypesField)

    def __init__(self,*k,**kw):
        super(ManagePloneBatchBase,self).__init__(*k,**kw)
        self._btypes_utility = zapi.getUtility(IBatchEditableContentTypes)

    @button.buttonAndHandler(_(u'Save'))
    def handleSave(self, action):
        """Mark batch-editable content types"""
        context = aq_inner(self.context)
        form_prefix = 'form.widgets.'
        form = self.request.form
        bfield = self.fields['batch_editable_types'].field
        datamanager = getMultiAdapter((self.context,bfield),IDataManager)
        datamanager.set(form.get(form_prefix+'batch_editable_types',[]))
        IStatusMessage(self.request).addStatusMessage(_(u'Changes Saved'),
                                                      type='info')
        redirect_to = '%s/@@manage_plone_batch' % context.absolute_url()
        self.request.response.redirect(redirect_to)


class ManagePloneBatch(FormWrapper):
    form = ManagePloneBatchBase
    label = _(u'Plone Batch Set Up')
    description = _(u'Set up plone batch options')
    index = ViewPageTemplateFile('templates/plone_setup_skeleton.pt')

class BatchSetupDataManager(DataManager):
    """Batch editable content types data manager."""
    adapts(Interface, IBatchContentTypesField)

    def __init__(self, context, field):
        self.context = context
        self.field = field
        self._btypes_utility = zapi.getUtility(IBatchEditableContentTypes)

    def get(self):
        """See z3c.form.interfaces.IDataManager"""
        context = self.context
        btypes = self._btypes_utility.batch_editable_types(self.context)
        return [type.getId() for type in btypes]

    def query(self, default=NOVALUE):
        """See z3c.form.interfaces.IDataManager"""
        try:
            return self.get()
        except:
            return default

    def set(self, value):
        """See z3c.form.interfaces.IDataManager"""
        if self.field.readonly:
            raise TypeError("Can't set values on read-only fields "
                            "(name=%s, class=%s.%s)"
                            % (self.field.__name__,
                               self.context.__class__.__module__,
                               self.context.__class__.__name__))
        context = self.context
        for type in self._btypes_utility.all_types(context):
            in_new_value = unicode(type.getId()) in value
            marked = IBatchEditableMarker.providedBy(type)
            if in_new_value and not marked:
                marker.mark(type,IBatchEditableMarker)
            if not in_new_value and marked:
                marker.erase(type,IBatchEditableMarker)

    def canAccess(self):
        """See z3c.form.interfaces.IDataManager"""
        return True

    def canWrite(self):
        """See z3c.form.interfaces.IDataManager"""
        return True
