from Acquisition import aq_inner
from zope.interface import alsoProvides

from plone.z3cform.crud import crud
from plone.memoize.instance import memoize

from plone.app.batch.interfaces import IBatchField

_special_nothing = object()

class BatchBaseForm(object):
    def getConfigurations(self,key=None,default=_special_nothing):
        """Returns configurations of context for the given key. plone.app.batch
        stores different configurations on sessions."""
        if default is _special_nothing:
            default = {}
        if hasattr(self,'_batch_context'):
            context = aq_inner(self._batch_context)
        else:
            context = aq_inner(self.context)
        session = self.request['SESSION']
        context_id = 'plone.app.batch %s' % context.absolute_url_path()
        if not session.has_key(context_id):
            configuration = {}
            session.set(context_id,configuration)
        else:
            configuration = session.get(context_id)

        if key is None:
            return configuration
        else:
            return configuration.setdefault(key,default)

class BatchCrudFormBase(crud.CrudForm,BatchBaseForm):
    addform_factory = crud.NullForm
    _editform_factory = None # put there the factory
    _prev_step = 'batch_prev_step' # the name of the previous form step here
    _this_step = 'batch_this_step' # and of the current form step here

    def __init__(self,*k,**kw):
        super(BatchCrudFormBase,self).__init__(*k,**kw)
        self.editform_factory = self.EditFactory
        url_keys = [k for k,v in self.request.items()
                      if isinstance(v,str) and self._prev_step in v]
        for k in url_keys:
            v = self.request.get(k).replace(self._prev_step,self._this_step)
            self.request.set(k,v)
        self.items = self.initial_items()

    def EditFactory(self,*k,**kw):
        kw['items'] = self.items
        kw['_batch_context'] = self.context
        return self._editform_factory(*k,**kw)

class BatchInnerEditFormBase(crud.EditForm,BatchBaseForm):
    label = u''
    _subeditform_factory = crud.EditSubForm

    def __init__(self,*k,**kw):
        for attr,default in [('items',{}),('_batch_context',None)]:
            try:
                value = kw.pop(attr)
            except KeyError:
                value = default
            setattr(self,attr,value)
        super(BatchInnerEditFormBase,self).__init__(*k,**kw)

    @memoize
    def needing_fixes(self):
        """Returns the subforms that are needing some fixes before commiting
        changes."""
        return filter(lambda x: getattr(x,'need_fix',False), self.subforms)

    @memoize
    def no_needing_fixes(self):
        """Returns the subforms that are not needing fixes"""
        return filter(lambda x: x not in self.needing_fixes(), self.subforms)

    def _update_subforms(self):
        self.subforms = []
        self.missing_attrs = []
        for id, item in self.context.get_items():
            subform = self._subeditform_factory(self, self.request)
            subform.content = item
            subform.content_id = id
            try:
                subform.update()
            except AttributeError, e:
                # In case the content to edit does not have the field to edit,
                # we expect strings like:
                # "'BatchEditable' object has no attribute 'text'"
                prefix = "'BatchEditable' object has no attribute '"
                if not str(e).startswith(prefix):
                    raise
                key = str(e).replace(prefix,'').replace('\'','')
                tha_names = reduce(list.__add__,[ff.keys()
                                            for ff in subform._tha_fields()])
                if not key in tha_names:
                    raise
                self.missing_attrs.append((item,key))
            else:
                self.subforms.append(subform)

class BatchSubEditFormBase(crud.EditSubForm,BatchBaseForm):

    def __init__(self,*k,**kw):
        super(BatchSubEditFormBase,self).__init__(*k,**kw)
        self._batch_context = self.context._batch_context

    @property
    def fields(self):
        fields = super(BatchSubEditFormBase,self).fields
        for f in fields.values()[1:]:
            # Note that we skipped the first one, because is the select
            # checkbox, and we don't want to mark it with IBatchField
            alsoProvides(f.field,IBatchField)
        return fields
