from zope.interface import Interface, implements, alsoProvides
from zope.component import adapts
from zope.app.pagetemplate.viewpagetemplatefile import ViewPageTemplateFile as zViewPageTemplateFile
from zope.schema.interfaces import IDatetime

from z3c.form import button
from z3c.form import field
from z3c.form.interfaces import DISPLAY_MODE

from z3c.form.interfaces import INPUT_MODE, NOVALUE, IDataManager
from z3c.form.datamanager import AttributeField

from plone.app.z3cform.layout import FormWrapper
from plone.z3cform.crud import crud
from plone.memoize.instance import memoize

from Products.CMFCore.utils import getToolByName

from plone.app.batch import BatchMessageFactory as _
from plone.app.batch import utils
from plone.app.batch.interfaces import IBatchField
from plone.app.batch.browser import baseform
from plone.app.batch.browser.interfaces import IBatchEditorSubForm
from plone.app.batch.browser.interfaces import IBatchEditable
from plone.app.batch.browser.fields import IReference
from plone.app.batch.browser.fields import ILines

class BatchDataManager(AttributeField):
    """Data Manager for batch fields."""
    adapts(Interface, IBatchField)

    def get(self):
        """See z3c.form.interfaces.IDataManager"""
        value = super(BatchDataManager,self).get()
        if IReference.providedBy(self.field):
            value = [v.UID() for v in value]
        if IDatetime.providedBy(self.field):
            value = utils.to_datetime(value)
        if ILines.providedBy(self.field):
            # FIXME: Archetype Line fields are treated as text zope fields so far
            value = '\n'.join(value)
        if isinstance(value,str):
            value = unicode(value,'utf-8')
        return value

    def set(self,value):
        """See z3c.form.interfaces.IDataManager"""
        if IDatetime.providedBy(self.field):
            value = utils.to_DateTime(value)
        super(BatchDataManager,self).set(value)

class BatchEditable(object):
    own_attr_names = ['context','fields_accessors','fields_mutators']

    def __init__(self,context):
        self.context = context
        self.fields_accessors = dict([(f.getName(),f.accessor)
                                        for f in self.context.schema.fields()])
        self.fields_mutators = dict([(f.getName(),f.mutator)
                                        for f in self.context.schema.fields()])

    def __getattr__(self,key):
        if key in BatchEditable.own_attr_names:
            return object.__getattribute__(self,key)
        elif key in IBatchEditable.names()+self.fields_accessors.keys():
            return getattr(self.context,self.fields_accessors[key])()
        else:
            return object.__getattribute__(self,key)

    def __setattr__(self,key,value):
        if key in BatchEditable.own_attr_names:
            return object.__setattr__(self,key,value)
        elif key in IBatchEditable.names()+self.fields_mutators.keys():
            set_result = getattr(self.context,self.fields_mutators[key])(value)
            self.context.reindexObject()
            return set_result
        else:
            return object.__setattr__(self,key,value)

class EditorSubForm(baseform.BatchSubEditFormBase):
    implements(IBatchEditorSubForm)

    def _select_field(self):
        """Returns the select field but putting true as default"""
        select_field = super(EditorSubForm,self)._select_field()
        select_field.field.default = True
        return select_field

    def _tha_fields(self):
        """Returns fields the user selected to batch-edit"""
        return [field.Fields(f)
                             for f in self.getConfigurations('fields').values()]

    @property
    def fields(self):
        fields = super(EditorSubForm,self).fields
        for f in self._tha_fields():
            fields += f
        return fields

class InnerEditorForm(baseform.BatchInnerEditFormBase):
    template = zViewPageTemplateFile('templates/crud-table.pt')
    _subeditform_factory = EditorSubForm

    def remove_subform(self,subform):
        item = subform.content
        configurations = self.getConfigurations()
        current_items = configurations['items']
        if current_items.get(item.id,None) == item.absolute_url_path():
            current_items.pop(item.id)
        configurations['items'] = current_items
        self.subforms.remove(subform)

    @button.buttonAndHandler(_('Save'), name='save')
    def handle_edit(self, action):
        success = _(u"Changes Saved.")
        no_changes = _(u"No changes made. Fix the errors for the objects displayed and try again.")
        status = no_changes
        data_extracted = {}
        error_happened = False
        for subform in self.subforms[:]:
            data,error = subform.extractData()
            data_extracted[subform.content.absolute_url_path()] = (data,error)
            if data['select']:
                subform.need_fix = bool(error)
                error_happened = error_happened or bool(error)
            else:
                self.remove_subform(subform)

        if error_happened: # damned, some error occured.
            hidden = _(u"There are %i object/s with correct data that will be submitted.")
            self.status = no_changes + '\n' + hidden % len(self.no_needing_fixes())
            return
        for subform in self.subforms:
            data, errors = data_extracted[subform.content.absolute_url_path()]
            del data['select']
            self.context.before_update(subform.content, data)
            changes = subform.applyChanges(data)
            if changes:
                [w.update() for w in subform.widgets.values()
                 if w.mode == DISPLAY_MODE]
            status = success
        self.status = status

class BatchEditorFormBase(baseform.BatchCrudFormBase):
    update_schema = IBatchEditable
    _editform_factory = InnerEditorForm
    _prev_step = 'batch_selector_form'
    _this_step = 'batch_editor_form'

    def initial_items(self):
        portal = getToolByName(self.context,'portal_url').getPortalObject()
        items = [(name,portal.restrictedTraverse(path))
                       for name,path in self.getConfigurations('items').items()]
        return dict(items)

    def get_items(self):
        return sorted(self.items.items(), key=lambda x: x[1].title)

class BatchEditorForm(FormWrapper):
    form = BatchEditorFormBase
    label = _(u"Batch edit form")
