from zope import schema
from zope.schema.interfaces import IField
from zope.schema.interfaces import ITuple
from zope.schema.interfaces import IText
from zope.interface import Interface

from z3c.form.interfaces import ISequenceWidget
from z3c.form.interfaces import ITextWidget
from z3c.form.interfaces import IForm

from plone.app.batch import BatchMessageFactory as _

class IReference(ITuple):
    """Reference field marker"""

class ILines(IText):
    """Lines field marker. Now from IText,
    but in future should inherit form ITuple"""

class IBatchContentTypesField(IField):
    """Marker interface for batchable-content-types fields"""

class ITranslatableContentTypeMultiCheckBoxWidget(ISequenceWidget):
    """Marker interface for translatable select field widgets"""

class IAccessibleSelectWidget(ISequenceWidget):
    """Marker interface for translatable select field widgets"""

class ITranslatableContentTypeTextWidget(ITextWidget):
    """Marker interface for translatable text field widgets"""

class IPloneBatchManagementContentTypes(Interface):
    """Marker interface that defines Batch editable objects.
    """
    batch_editable_types = schema.List(
        title = _(u"Batch-editable Types"),
        required = False,
        default = [],
        description = _(u"Select content types that will be able to be batch-editable"),
        value_type = schema.Choice(title=u"batch_editable_types",
                                   source="plone.content_types"))

class IBatchSelectable(Interface):
    """Interface of subforms displayed in selector form"""
    content_type = schema.TextLine(title=_(u'Type'),readonly=True,required=False)
    id = schema.TextLine(title=_(u'Id'),readonly=True,required=False)
    title = schema.TextLine(title=_(u'Title'),readonly=True,required=False)
    modified = schema.Datetime(title=_(u'Modified'),
                               readonly=True,required=False)

_batchable_types_choice = schema.Choice(title=u"portal_type",
                                source="plone.batch_editable_types")

class IBatchSearchForm(Interface):
    """Interface for Batch Search form"""
    SearchableText = schema.TextLine(title=_(u'Search Text'),required=False,
                     description=_(u"Enter your search term here. The text in this field will be matched with items' contents, title and description. Multiple words may be found by combining them with AND and OR."))
    Title = schema.TextLine(title=_(u'Title'),required=False,
                            description=_(u'Return items matching this title.'))
    Description = schema.TextLine(title=_(u'Description'),required=False,
                                  description=_(u'Return items matching this description. Multiple words may be found by combining them with AND and OR.'))
    portal_type = schema.List(
        title = _(u'Item Type'),
        required = False,
        default = [],
        description = _(u'Return items of a specific type.'),
        value_type = _batchable_types_choice)

class IBatchEditorSubForm(IForm):
    """Marker interface for Batch Editor Sub Forms"""

class IBatchEditable(Interface):
    """Almost empty interface that's used for creating crud batch-editing forms"""
    id = schema.TextLine(title=_(u'Id'),readonly=True,required=True)
