from Acquisition import aq_inner
from z3c.form import button
from z3c.form.interfaces import DISPLAY_MODE
from plone.app.z3cform.layout import FormWrapper
from zope.app.pagetemplate.viewpagetemplatefile import ViewPageTemplateFile as zViewPageTemplateFile

from Products.CMFCore.utils import getToolByName

from plone.app.batch import utils
from plone.app.batch import BatchMessageFactory as _
from plone.app.batch.interfaces import IAT2Z3Field
from plone.app.batch.browser.interfaces import IBatchSelectable
from plone.app.batch.browser import baseform
from plone.app.batch.browser.widgets import ContentTypeTextWidgetFactory
from plone.app.batch.utils import makeBatchEditableField

class BatchSelectable(object):

    def __init__(self,context):
        self.context = context

    def getId(self):
        return self.context.getId()
    id = property(getId)

    def getTitle(self):
        return unicode(self.context.Title(),'utf-8')
    title = property(getTitle)

    def getModified(self):
        context_date = self.context.modification_date
        context_date = utils.to_datetime(context_date)
        return context_date
    modified = property(getModified)

    def getContentType(self):
        return self.context.portal_type
    content_type = property(getContentType)

    def portal_type_info(self,type):
        context = aq_inner(self.context)
        portal_types = getToolByName(context,'portal_types')
        return portal_types.getTypeInfo(type)

    def getTranslatedContentType(self):
        print self, self.content_type
        return self.portal_type_info(self.content_type).Title()
    translated_content_type = property(getTranslatedContentType)

class SelectorSubForm(baseform.BatchSubEditFormBase):

    @property
    def fields(self):
        fields = super(SelectorSubForm,self).fields
        fields['content_type'].widgetFactory[DISPLAY_MODE] = ContentTypeTextWidgetFactory
        return fields

class InnerSelectorForm(baseform.BatchInnerEditFormBase):
    template = zViewPageTemplateFile('templates/crud-table.pt')
    _subeditform_factory = SelectorSubForm

    def __init__(self,*k,**kw):
        super(InnerSelectorForm,self).__init__(*k,**kw)
        self.item_fields = dict([(f.getName(),f) for obj in self.items.values()
                                                 for f in obj.schema.fields()])
        # Editing the id field of an object is actually renaming it. We don't
        # want to allow that from here
        self.item_fields.pop('id')

    def get_batch_editable_fields(self):
        return sorted(self.item_fields)

    @button.buttonAndHandler(_(u'Edit Field'), name='edit_field')
    def handle_edit_field(self, action):
        self.status = ''
        selected_items = self.selected_items()
        selected_field = self.request.form.get('batch_selected_field','')
        if not selected_items:
            self.status += _(u'Please select items to edit first. ')
        if not selected_field or selected_field not in self.item_fields:
            self.status += _(u'Please select the field to edit first. ')
        if selected_items and selected_field:
            context = aq_inner(self._batch_context)
            configurations = self.getConfigurations()
            configurations['items'] = dict([(id,ob.absolute_url_path())
                                                for id,ob in selected_items])

            # From now on, we assume that can exist more than one field to edit.
            # Previous work need to be made in previuos forms.
            selected_fields = [selected_field]
            items = map(lambda x: x[1],selected_items)

            z3fields = {}
            for fn in selected_fields:
                z3f = IAT2Z3Field( (self.item_fields[fn],items) )
                makeBatchEditableField(z3f)
                z3fields[fn] = z3f

            configurations['fields'] = z3fields

            redirect_to = '%s/@@batch_editor_form' % context.absolute_url()
            self.request.response.redirect(redirect_to)

class BatchSelectorFormBase(baseform.BatchCrudFormBase):
    update_schema = IBatchSelectable
    _editform_factory = InnerSelectorForm
    _prev_step = 'batch_search_form'
    _this_step = 'batch_selector_form'

    def initial_items(self):
        return dict([(o.id,o.getObject()) for o in self.process_search()])

    def process_search(self):
        context = aq_inner(self.context)
        use_types_blacklist = self.request.form.get('use_types_blacklist',True)
        use_navigation_root = self.request.form.get('use_navigation_root',True)
        form_request = self.request.clone()
        form_request.form = dict(self.getConfigurations('form'))
        form_request.form.update(self.request.form)
        form_request.form['path'] = context.absolute_url_path()
        results = context.queryCatalog(REQUEST=form_request,
                                       use_types_blacklist=use_types_blacklist,
                                       use_navigation_root=use_navigation_root)
        return results

    def get_items(self):
        return sorted(self.items.items(), key=lambda x: x[1].title)

class BatchSelectorForm(FormWrapper):
    form = BatchSelectorFormBase
    label = _(u"Batch search results")

