from Acquisition import aq_inner

from z3c.form.browser.checkbox import CheckBoxWidget
from z3c.form.browser.orderedselect import OrderedSelectWidget
from z3c.form.browser.text import TextWidget
from z3c.form.widget import FieldWidget
from z3c.form.interfaces import IFormLayer
from z3c.form.interfaces import IFieldWidget
from zope.schema.interfaces import IChoice
from zope.schema.interfaces import IField
from zope.component import adapter
from zope.interface import implementer, implementsOnly, implements

from Products.CMFCore.utils import getToolByName

from plone.app.batch.browser.interfaces import ITranslatableContentTypeMultiCheckBoxWidget
from plone.app.batch.browser.interfaces import ITranslatableContentTypeTextWidget
from plone.app.batch.browser.interfaces import IAccessibleSelectWidget
from plone.app.batch.browser.interfaces import IReference

class TranslatableContentWidgetBase(object):
    """Base class for translatable content type widgets"""
    def portal_type_info(self,type):
        context = aq_inner(self.context)
        portal_types = getToolByName(context,'portal_types')
        return portal_types.getTypeInfo(type)

class TranslatableContentTypeMultiCheckBoxWidget(CheckBoxWidget,TranslatableContentWidgetBase):
    implementsOnly(ITranslatableContentTypeMultiCheckBoxWidget)

class TranslatableContentTypeTextWidget(TextWidget,TranslatableContentWidgetBase):
    implementsOnly(ITranslatableContentTypeTextWidget)

class AccessibleSelectWidget(OrderedSelectWidget,TranslatableContentWidgetBase):
    implements(IAccessibleSelectWidget)

@adapter(IReference, IFormLayer)
@implementer(IFieldWidget)
def AccesibleOrderedSelectWidgetFactory(field, request):
    """IFieldWidget factory for TextWidget."""
    return FieldWidget(field, AccessibleSelectWidget(request))

def ContentTypesMultiCheckBoxWidgetFactory(field, request):
    """IFieldWidget factory for TranslatableContentTypeMultiCheckBoxWidget."""
    widget = TranslatableContentTypeMultiCheckBoxWidget(request)
    widget.multiple = 'multiple'
    return FieldWidget(field, widget)

def ContentTypeTextWidgetFactory(field, request):
    """IFieldWidget factory for TranslatableContentTypeTextWidget."""
    return FieldWidget(field, TranslatableContentTypeTextWidget(request))

