from zope.schema import vocabulary
from zope.interface import implements

from Products.CMFCore.utils import getToolByName

from plone.app.batch import BatchMessageFactory as _
from plone.app.batch import interfaces

class TitledVocabulary(vocabulary.SimpleVocabulary):
    def fromTitles(cls, items, *interfaces):
        """Construct a vocabulary from a list of (value,title) pairs.

        The order of the items is preserved as the order of the terms
        in the vocabulary.  Terms are created by calling the class
        method createTerm() with the pair (value, title).

        One or more interfaces may also be provided so that alternate
        widgets may be bound without subclassing.
        """
        terms = [cls.createTerm(value,value,title) for (value,title) in items]
        return cls(terms, *interfaces)
    fromTitles = classmethod(fromTitles)

def _searchable_content_types(context):
    portal_types = getToolByName(context, 'portal_types')
    properties = getToolByName(context, 'portal_properties')
    types_not_searched = set( properties.site_properties.types_not_searched )
    return [type for type in portal_types.objectValues()
                 if type.getId() not in types_not_searched]

def ContentTypes(context):
    terms = []
    for type in _searchable_content_types(context):
        terms.append(vocabulary.SimpleTerm(unicode(type.getId()),
                                           title=unicode(type.title_or_id())))
    terms.sort(lambda x,y:cmp(x.title,y.title))
    return vocabulary.SimpleVocabulary(terms)

class BatchEditableContentTypes(object):
    """Batch-editable content types"""
    implements(interfaces.IBatchEditableContentTypes)

    def batch_editable_types(self,context):
        return [type for type in _searchable_content_types(context) if 
                interfaces.IBatchEditableMarker.providedBy(type)]

    def all_types(self,context):
        return _searchable_content_types(context)

def BatchEditableContentTypesVocabulary(context):
    bect = BatchEditableContentTypes()
    terms = []
    for type in bect.batch_editable_types(context):
        terms.append(vocabulary.SimpleTerm(unicode(type.getId()),
                                        title=unicode(type.title_or_id())))
    terms.sort(lambda x,y:cmp(x.title,y.title))
    return vocabulary.SimpleVocabulary(terms)
